%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2020 Biosemi(Gerben Snoek <info@biosemi.com>) <www.biosemi.com> 
%
%    This source code is free software; you can redistribute it
%    and/or modify it in source code
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   This program is written to provide a matlab example that demonstrate
%   the reading and importing of a BDF file
%   This example is made with matlab R2014a
%   You have to call this function with the filename, Chunk (seconde) and
%   channel that you want to see
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Header, Data] = ImportBDF_Biosemi(Filename,chunk,channel) % By default is 1 sec == 1 chunk in a bdf file

FileId = fopen(Filename, 'r', 'l');        % open file as readonly in little endian mode
    if FileId < 0                          % check if the file is opend correctly    
        error('File not found')
    end
    
Header = ImportBDFHeader_Biosemi(FileId);  % Decode the header of the BDF file. If you want multiple channels or chunks from the same file you only have to decode the header once.

    % check if the user input is valide for this file
    if (chunk > Header.DataChunks) || (channel > Header.Channels) || (chunk < 1) || (channel < 1)
        error('channel or chunk is not available in this file')
    end

[startposition, size] = CalculateBlockSize(Header, chunk, channel); % calculate the location and size of the requested data.

Data = ImportBDFData_Biosemi(FileId, startposition, size );

%calculate uV from raw data LSB = 31.25nV
Data = Data / 1000 * 31.25 ;

fclose(FileId);

%plot the data
plot(Data);
title(Header.Channel{channel}.Lables);
xlabel('Samples');
ylabel('uV');

end


function [startposition, size] = CalculateBlockSize (Header, chunk, channel)

% calculate the size of a single chunk by summing the sampelrate of all the channels * 3 bytes per sample
chunksize = 0;
    for ChannelNumber = 1:Header.Channels
        chunksize = chunksize + ( Header.Channel{ChannelNumber}.SampleRate *3 );  
    end

% calculate the channel offset within a chunk by summing the channels before the selected channel
ChannelOffset = 0;
    if channel == 1
        % first channel no offset nessesary
    else
        for ChannelNumber = 1:(channel-1)
            ChannelOffset = ChannelOffset + (Header.Channel{ChannelNumber}.SampleRate * 3);
        end 
    end

%calculate the position of the first byte of the selected channel within the selected chunk
startposition = Header.headersize  + (chunksize*(chunk-1)) + ChannelOffset;

size = Header.Channel{channel}.SampleRate;

end


function Data = ImportBDFData_Biosemi(FileId,startposition,DataSize)

fseek(FileId, startposition, 'bof');
Data = fread(FileId,DataSize,'bit24');

end


function Header = ImportBDFHeader_Biosemi(FileId) %Decodes the header of a Biosemi BDF file

frewind(FileId);

%check if the file is a BDF file
version = fread(FileId,8,'uint8')';
identify = char(version(2:8));
    if (version(1) == 255) && strcmp( identify , 'BIOSEMI')
        %disp('The file is a BDF file')
    else
        error('This file is not a BDF file')
    end

Header.SubjectId          = char(fread(FileId,80,'char')');              % Local subject identification
Header.RecDescription     = char(fread(FileId,80,'char')');              % Local recording identification
Header.StartDate          = char(fread(FileId,8,'char')');               % Startdate of recording
Header.StartTime          = char(fread(FileId,8,'char')');               % Starttime of recording
Header.headersize         = str2num(char(fread(FileId,8,'char')'));      % Number of bytes in header record
Header.DataFormat         = char(fread(FileId,44,'char')');              % Version of data format.
Header.DataChunks         = str2num(char(fread(FileId,8,'char')'));      % Number of data records "-1" if unknown
Header.ChunkDuration      = str2num(char(fread(FileId,8,'char')'));      % Duration of a data record, in seconds
Header.Channels           = str2num(char(fread(FileId,4,'char')'));      % Number of channels (N) in data record

    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.Lables = char(fread(FileId,16,'char')');               % Labels of the channels
    end
    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.Type = char(fread(FileId,80,'char')');                 % Transducer type 
    end
    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.Unit = char(fread(FileId,8,'char')');                  % Physical dimension of channels
    end
    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.PhyMin = str2num(char(fread(FileId,8,'char')'));       % Physical minimum in units of physical dimension
    end
    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.PhyMax = str2num(char(fread(FileId,8,'char')'));       % Physical maximum in units of physical dimension
    end
    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.DigMin = str2num(char(fread(FileId,8,'char')'));       % Digital minimum
    end
    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.DigMax = str2num(char(fread(FileId,8,'char')'));       % Digital maximum
    end
    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.PreFilter = char(fread(FileId,80,'char')');            % Prefiltering
    end
    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.SampleRate = str2num(char(fread(FileId,8,'char')'));   % Number of samples in each data record (Sample-rate if Duration of data record = "1")
    end
    for ChannelNumber = 1:Header.Channels
        Header.Channel{ChannelNumber}.Reserved = char(fread(FileId,32,'char')');             % Reserved
    end

end

